@@ -205,6 +205,11 @@ public:
     /**@return a pointer to the current Altitude. */
     inline const dms& alt() const { return Alt; }
 
+    /**@return refracted altitude. This function uses
+     * Option::useRefraction to determine whether refraction
+     * correction should be aplied */
+    dms altRefracted() const;
+
     //XYZ
     inline const Quaternion& quat() const { return m_q; }
     void syncQuaternion();
@@ -371,7 +376,7 @@ public:
      *  @param sp SkyPoint to which distance is to be calculated
      *  @return dms angle representing angular separation.
      **/
-    dms angularDistanceTo(const SkyPoint *sp);
+    dms angularDistanceTo(const SkyPoint *sp) const;
 
     inline bool operator == ( SkyPoint &p ) { return ( ra() == p.ra() && dec() == p.dec() ); }
 
@@ -463,12 +468,22 @@ public:
      */
     SkyPoint moveAway( SkyPoint &from, double dist );
 
-
     /**
      * @short Check if this point is circumpolar at the given geographic latitude
      */
     bool checkCircumpolar( const dms *gLat );
 
+    /** Calculate refraction correction. Parameter and return value are in degrees */
+    static double refractionCorr( double alt );
+
+    /** Apply refraction correction to altitude. */
+    static dms refract(dms h);
+
+    /** Remove refraction correction. */
+    static dms unrefract(dms h);
+
+    static const double altCrit  = -1;
+
 protected:
     /**Precess this SkyPoint's catalog coordinates to the epoch described by the
     	*given KSNumbers object.
