@@ -1,7 +1,7 @@
 /***************************************************************************
  *   ktouch.h                                                              *
  *   --------                                                              *
- *   Copyright (C) 2000 by Håvard Frøiland, 2004 by Andreas Nicolai        *
+ *   Copyright (C) 2000 by Håvard Frøiland, 2003-2007 by Andreas Nicolai   *
  *   ghorwin@users.sourceforge.net                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -13,32 +13,26 @@
 #ifndef KTOUCH_H
 #define KTOUCH_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <QLabel>
+#include <QKeyEvent>
+#include <QResizeEvent>
 
-#include <qcolor.h>
-#include <qstringlist.h>
-#include <qvaluevector.h>
-
-#include <kmainwindow.h>
-#include <kapplication.h>
+#include <kxmlguiwindow.h>
 #include <kurl.h>
 
-class QLabel;
-class KToggleAction;
-class KActionMenu;
+class KAction;
 class KSelectAction;
 
-class KTouchStatus;
-class KTouchSlideLine;
+class KTouchStatusWidget;
+class KTouchTextLineWidget;
 class KTouchKeyboardWidget;
 class KTouchTrainer;
 
-class KTouchPrefTrainingLayout;
-class KTouchPrefKeyboardLayout;
-class KTouchPrefGeneralLayout;
-class KTouchPrefColorsLayout;
+class KTouchPrefTraining;
+class KTouchPrefKeyboard;
+class KTouchPrefGeneral;
+class KTouchPrefColors;
+class KTouchPrefSound;
 
 #include "ktouchlecture.h"
 #include "ktouchstatisticsdata.h"
@@ -56,7 +50,7 @@ class KTouchPrefColorsLayout;
 /// typed char. It delegates the character to the trainer
 /// (KTouchTrainer), which will then process it. So the heavy work lies in the trainer
 /// object and all the widgets.
-class KTouch : public KMainWindow {
+class KTouch : public KXmlGuiWindow {
     Q_OBJECT
   public:
     /// Constructor, creates the KTouch proggy.
@@ -66,6 +60,8 @@ class KTouch : public KMainWindow {
 
 	/// Returns the available lecture files
 	const QStringList& lectureFiles() const { return m_lectureFiles; }
+	/// Returns the available keyboard files
+	const QStringList& keyboardFiles() const { return m_keyboardFiles; }
 	/// Returns the statistics object for the current lecture (as reference)
 	KTouchLectureStats& getCurrentLectureStats();
 	/// Clears the statistics data.
@@ -76,20 +72,20 @@ class KTouch : public KMainWindow {
 	void changeStatusbarStats(unsigned int level_correct, unsigned int level_total, unsigned int level_words,
 							  unsigned int session_correct, unsigned int session_total, unsigned int session_words);
 
+	///< Returns list of user names.
+    const QStringList& userList() const { return m_userList; }
+	///< A map with statistics for each user.
+	const QMap<QString, KTouchStatisticsData>& userStats() const { return m_userStats; }
+
+
   public slots:
     /// Will be called when the "Apply"-button has been pressed in the preferences
     /// dialog or when the user accepted the changes using the "OK"-button.
     void applyPreferences();
+    /// Accepts a typed char.
+    void keyPressEvent(QKeyEvent *keyEvent);
 
-	/// Called from the configuration dialog.
-	void configOverrideLectureFontToggled(bool on);
-	/// Called from the configuration dialog. 
-	void configOverrideKeyboardFontToggled(bool on);
-	/// Called from the configuration dialog.
-	void configAutoLevelChangeToggled(bool on);
-	/// Called from the configuration dialog.
-	void configCommonColorsToggled(bool on);
-
+	void fileOpenText();                ///< The action File->Open text...
     void fileOpenLecture();             ///< The action File->Open lecture...
     void fileEditLecture();             ///< The action File->Edit lecture...
     void fileEditColors();              ///< The action File->Edit colors...
@@ -98,7 +94,8 @@ class KTouch : public KMainWindow {
     void trainingNewSession();          ///< The action Training->Start new training session...
     void trainingPause();               ///< The action Training->Pause training
     void trainingStatistics();          ///< The action Training->Show training statistics...
-    void optionsPreferences();          ///< The action Settings->Configure KTouch...
+    void optionsPreferences();          ///< The action Options->Configure KTouch...
+	void optionsSetupUsers();           ///< The action Options->Setup users...
 
     /// Quick-changes the keyboard layout (called from menu).
     void changeKeyboard(int num);
@@ -106,25 +103,18 @@ class KTouch : public KMainWindow {
     void changeColor(int num);
     /// Quick-changes the current training lecture file (called from menu).
     void changeLecture(int num);
+    /// Quick-changes the current user (called from menu).
+    void changeUser(int num);
 
   protected:
     /// Reimplementated to save preferences and
     bool queryExit();
     /// Some layout fixes here...
-	void resizeEvent(QResizeEvent *);
-    /// Accepts a typed char.
-    void keyPressEvent(QKeyEvent *keyEvent);
+    void resizeEvent(QResizeEvent *);
 
-    void imEndEvent (QIMEvent *e);
+    void inputMethodEvent( QInputMethodEvent* m );
 
   private:
-    // *** BEGIN - Session management ***
-    /// Will be called when this app is restored due to session management.
-    void readProperties(KConfig *config);
-    /// Will be called when the app should save its state for session management purposes.
-    void saveProperties(KConfig *config);
-    // *** END - Session management ***
-	
     /// Initialises the program during a normal startup
 	void init();
 	/// Creates the layout and GUI setup for a practice session
@@ -139,31 +129,37 @@ class KTouch : public KMainWindow {
 	/// This function populates the file lists with the installed training, keyboard and 
 	/// examination files.
     void updateFileLists();
-	/// Updates the check mark in the lecture-quick-selection menu depending on the
+	/// Updates the check mark in the lecture-quick-selection menu depending on the 
 	/// lecture in Prefs::currentLectureFile().
 	void updateLectureActionCheck();
 	/// Updates the check mark in the keyboard-quick-selection menu depending on the 
 	/// lecture in Prefs::currentKeyboardFile().
 	void updateKeyboardActionCheck();
-	
+	/// Updates the check mark in the user-quick-selection menu depending on the
+	/// user name in Prefs::currentUserName().
+	void updateCurrentUserActionCheck();
+
+
     // *** Public member variables ***
-    KAction                *m_trainingPause;        ///< Action for "pause training session".
+    QAction                *m_trainingPause;        ///< Action for "pause training session".
 
     KSelectAction		   *m_keyboardLayoutAction;
     KSelectAction		   *m_keyboardColorAction;
     KSelectAction		   *m_defaultLectureAction;
+    KSelectAction		   *m_currentUserAction;
 
-    KTouchStatus           *m_statusWidget;         ///< Pointer to the status widget on top of the main widget.
-    KTouchSlideLine        *m_slideLineWidget;      ///< Pointer to the sliding line widget.
+    KTouchStatusWidget     *m_statusWidget;         ///< Pointer to the status widget on top of the main widget.
+    KTouchTextLineWidget   *m_textLineWidget;       ///< Pointer to the sliding line widget.
     KTouchKeyboardWidget   *m_keyboardWidget;       ///< Pointer to the keyboard widget.
     KTouchTrainer          *m_trainer;              ///< The training 'master' (runs the training).
     KTouchLecture           m_lecture;              ///< The lecture data.
 
-	KTouchPrefGeneralLayout  * m_pageGeneral;		///< The general configuration page.
-	KTouchPrefTrainingLayout * m_pageTraining;		///< The training configuration page.
-	KTouchPrefKeyboardLayout * m_pageKeyboard;		///< The keyboard configuration page.
-	KTouchPrefColorsLayout   * m_pageColors;		///< The color scheme configuration page.
-	
+	KTouchPrefGeneral      *m_pageGeneral;          ///< The general configuration page.
+	KTouchPrefTraining     *m_pageTraining;         ///< The training configuration page.
+	KTouchPrefKeyboard     *m_pageKeyboard;         ///< The keyboard configuration page.
+	KTouchPrefColors       *m_pageColors;		    ///< The color scheme configuration page.
+    KTouchPrefSound        *m_pageSound;
+
     QStringList     		m_lectureFiles;         ///< A list of all default lecture files.
     QStringList     		m_lectureTitles;        ///< A list of the titles of all default lecture files.
 	
@@ -173,9 +169,11 @@ class KTouch : public KMainWindow {
     QStringList     		m_keyboardFiles;        ///< A list of all default keyboard layout files.
     QStringList     		m_keyboardTitles;       ///< A list of the titles of all default keyboard layout files.
 
-	KTouchStatisticsData	m_stats;				///< All user statistics are kept here.
+    QStringList    							m_userList; 		///< A list of user names.
+	QMap<QString, KTouchStatisticsData>		m_userStats;		///< A map with statistics for each user.
 
-	QChar					m_lastDeadKey;			///< Temporary storage of last dead key.
+    QLabel* statusBarLevel;
+    QLabel* statusBarSession;
 };
 
 /// A global pointer to the main widget (actually only used to retrieve some data).
