@@ -1,8 +1,8 @@
 /***************************************************************************
-                          kwqtableview.cpp  -  description
+                              kwqtableview.cpp
                              -------------------
     begin          : Wed Jul 24 20:12:30 PDT 2002
-    copyright      : (C) 2002-2009 Peter Hedlund <peter.hedlund@kdemail.net>
+    copyright      : (C) 2002-2010 Peter Hedlund <peter.hedlund@kdemail.net>
 
  ***************************************************************************/
 
@@ -17,6 +17,8 @@
 
 #include "kwqtableview.h"
 
+#include <QtCore/QFile>
+#include <QtCore/QPointer>
 #include <QPainter>
 #include <QClipboard>
 #include <QLineEdit>
@@ -25,7 +27,6 @@
 #include <QApplication>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
-#include <QtGui/QTextDocument>
 #include <QtGui/QTextCursor>
 #include <QtGui/QTextTable>
 #include <QtGui/QTextLayout>
@@ -38,12 +39,15 @@
 #include <KNotification>
 #include <KDebug>
 #include <kdeprintdialog.h>
+#include <KPrintPreview>
+#include <KFileDialog>
 
 #include "kwqtablemodel.h"
 #include "keduvocdocument.h"
 #include "kwordquiz.h"
 #include "prefs.h"
 #include "kwqcommands.h"
+#include "kwqcleardialog.h"
 
 //krazy:excludeall=qclasses
 
@@ -66,29 +70,62 @@ KWQTableView::KWQTableView(KUndoStack *undoStack, QWidget *parent) : QTableView(
 
 }
 
-void KWQTableView::print()
+void KWQTableView::doPrint()
 {
+  QPrinter printer;
+
   WQPrintDialogPage * p = new WQPrintDialogPage(this);
   p->setPrintStyle(Prefs::printStyle());
-  QPrinter printer(QPrinter::ScreenResolution);
   QPrintDialog *printDialog = KdePrint::createPrintDialog(&printer, QList<QWidget*>() << p, this);
-  printer.setFullPage(true);
-  if (printDialog->exec() != QDialog::Accepted)
-  {
-    delete printDialog;
-    return;
+  if (printDialog->exec() == QDialog::Accepted) {
+    Prefs::setPrintStyle(p->printStyle());
+    QTextDocument td;
+    createPages(&printer, &td, true);
   }
-  Prefs::setPrintStyle(p->printStyle());
+  delete printDialog;
+}
 
+void KWQTableView::doPrintPreview()
+{
+  QPrinter printer;
+  KPrintPreview preview(&printer, this);
   QTextDocument td;
+  createPages(&printer, &td, true);
+  preview.exec();
+}
+
+bool KWQTableView::doHtmlExport(const KUrl &url)
+{
+  bool success = false;
+
+  KUrl tmp(url);
+  QFile data(tmp.path());
+  if (data.open(QFile::WriteOnly)) {
+    QPrinter printer;
+    QTextDocument td;
+    td.setMetaInformation(QTextDocument::DocumentTitle, model()->sourceModel()->document()->title());
+    QTextStream out(&data);
+    out.setCodec("utf-8");
+    createPages(&printer, &td, false);
+    out << td.toHtml("utf-8");
+    data.close();
+    success = true;
+  }
+  return success;
+}
+
+void KWQTableView::createPages(QPrinter *printer, QTextDocument *textDoc, bool sendToPrinter)
+{
+  printer->setFullPage(true);
+  int myDpi = printer->logicalDpiY();
 
   if (Prefs::printStyle() == Prefs::EnumPrintStyle::Flashcard) {
-    printer.setOrientation(QPrinter::Landscape);
+    printer->setOrientation(QPrinter::Landscape);
 
-    int cardWidth = qRound(5 * printer.logicalDpiY());
-    int cardHeight = qRound(3 * printer.logicalDpiY());
+    int cardWidth = qRound(5 * myDpi);
+    int cardHeight = qRound(3 * myDpi);
 
-    QTextTable *table = td.rootFrame()->lastCursorPosition().insertTable(model()->rowCount(), 2);
+    QTextTable *table = textDoc->rootFrame()->lastCursorPosition().insertTable(model()->rowCount(), 2);
 
     QTextTableFormat tableFormat = table->format();
     tableFormat.setHeaderRowCount(0);
@@ -116,19 +153,19 @@ void KWQTableView::print()
     cellCharFormat.setFont(Prefs::editorFont());
 
     QTextFrameFormat cardFormat;
-    cardFormat.setBorder(logicalDpiY()/72);
+    cardFormat.setBorder(1);
     cardFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
     cardFormat.setBorderBrush(QBrush(Qt::black));
     cardFormat.setWidth(QTextLength(QTextLength::FixedLength, cardWidth));
     cardFormat.setHeight(QTextLength(QTextLength::FixedLength, cardHeight));
-    cardFormat.setPadding(qRound(0.25 * printer.logicalDpiY()));
+    cardFormat.setPadding(qRound(0.25 * myDpi));
 
     QTextFrameFormat lineFormat;
-    lineFormat.setBorder(logicalDpiY()/72);
+    lineFormat.setBorder(1);
     lineFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
     lineFormat.setBorderBrush(QBrush(Qt::black));
-    lineFormat.setWidth(QTextLength(QTextLength::FixedLength, qRound(4.5 * printer.logicalDpiY())));
-    lineFormat.setHeight(logicalDpiY()/38);
+    lineFormat.setWidth(QTextLength(QTextLength::FixedLength, qRound(4.5 * myDpi)));
+    lineFormat.setHeight(1.1); //1 is drawn as a box whereas this is drawn as a line. Strange...
     lineFormat.setPadding(0);
 
     QTextFrame *card;
@@ -147,31 +184,31 @@ void KWQTableView::print()
   }
   else
   {
-    td.rootFrame()->lastCursorPosition().insertText(KGlobal::caption());
+    textDoc->rootFrame()->lastCursorPosition().insertText(KGlobal::caption());
 
     if (Prefs::printStyle() == Prefs::EnumPrintStyle::Exam)
-      td.rootFrame()->lastCursorPosition().insertText(' ' + i18n("Name:_____________________________ Date:__________"));
+      textDoc->rootFrame()->lastCursorPosition().insertText(' ' + i18n("Name:_____________________________ Date:__________"));
 
     QTextTable* table;
     if (Prefs::printStyle() == Prefs::EnumPrintStyle::Exam)
-      table = td.rootFrame()->lastCursorPosition().insertTable(model()->rowCount() + 1, model()->columnCount() + 2);
+      table = textDoc->rootFrame()->lastCursorPosition().insertTable(model()->rowCount() + 1, model()->columnCount() + 2);
     else
-      table = td.rootFrame()->lastCursorPosition().insertTable(model()->rowCount() + 1, model()->columnCount() + 1);
+      table = textDoc->rootFrame()->lastCursorPosition().insertTable(model()->rowCount() + 1, model()->columnCount() + 1);
 
     QTextTableFormat tableFormat = table->format();
     tableFormat.setHeaderRowCount(1);
-    tableFormat.setBorder(logicalDpiY()/72 /*0.5*/);
-    kDebug() << logicalDpiY();
+    tableFormat.setBorder(1);
     tableFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
     tableFormat.setCellSpacing(0);
     tableFormat.setBorderBrush(QBrush(Qt::black));
     tableFormat.setCellPadding(2);
 
     QVector<QTextLength> constraints;
-    constraints.append(QTextLength(QTextLength::FixedLength, verticalHeader()->width() * printer.logicalDpiY()/logicalDpiY()));
-    constraints.append(QTextLength(QTextLength::FixedLength, columnWidth(0) * printer.logicalDpiY()/logicalDpiY()));
-    constraints.append(QTextLength(QTextLength::FixedLength, columnWidth(1) * printer.logicalDpiY()/logicalDpiY()));
-    constraints.append(QTextLength(QTextLength::FixedLength, 50 * printer.logicalDpiY()/logicalDpiY()));
+    constraints.append(QTextLength(QTextLength::FixedLength, verticalHeader()->width()));
+    constraints.append(QTextLength(QTextLength::FixedLength, columnWidth(0)));
+    constraints.append(QTextLength(QTextLength::FixedLength, columnWidth(1)));
+    if (Prefs::printStyle() == Prefs::EnumPrintStyle::Exam)
+        constraints.append(QTextLength(QTextLength::FixedLength, 50));
     tableFormat.setColumnWidthConstraints(constraints);
 
     table->setFormat(tableFormat);
@@ -211,10 +248,12 @@ void KWQTableView::print()
         table->cellAt(i + 1, 2).firstCursorPosition().insertText(model()->data(model()->index(i, 1)).toString(), cellCharFormat);
     }
   }
-  td.print(&printer);
-  delete printDialog;
+
+  if (sendToPrinter)
+    textDoc->print(printer);
 }
 
+
 void KWQTableView::doEditCut()
 {
   if (state() == QAbstractItemView::EditingState)
@@ -277,6 +316,21 @@ void KWQTableView::doEditClear()
   }
   else
   {
+    if (selectionHasMoreThanText()) {
+      QPointer<KWQClearDialog> clearDialog = new KWQClearDialog(this);
+      if (clearDialog->exec() == KDialog::Rejected) {
+        delete clearDialog;
+        return;
+      }
+      delete clearDialog;
+    }
+    else
+    {
+      Prefs::setClearAll(false);
+      Prefs::setClearText(true);
+      Prefs::setClearImageLink(false);
+      Prefs::setClearSoundLink(false);
+    }
     KWQCommandClear *kwqc = new KWQCommandClear(this);
     m_undoStack->push(kwqc);
   }
@@ -284,14 +338,18 @@ void KWQTableView::doEditClear()
 
 void KWQTableView::doEditInsert()
 {
-  KWQCommandInsert *kwqc = new KWQCommandInsert(this);
-  m_undoStack->push(kwqc);
+  if (selectionModel()->currentIndex().isValid()) {
+    KWQCommandInsert *kwqc = new KWQCommandInsert(this);
+    m_undoStack->push(kwqc);
+  }
 }
 
 void KWQTableView::doEditDelete()
 {
-  KWQCommandDelete *kwqc = new KWQCommandDelete(this);
-  m_undoStack->push(kwqc);
+  if (selectionModel()->currentIndex().isValid()) {
+    KWQCommandDelete *kwqc = new KWQCommandDelete(this);
+    m_undoStack->push(kwqc);
+  }
 }
 
 
@@ -653,6 +711,30 @@ void KWQTableView::slotHeaderClicked(int column)
 }
 
 
+void KWQTableView::doVocabImage()
+{
+  KUrl currentUrl = model()->data(currentIndex(), KWQTableModel::ImageRole).toString();
+
+  KUrl imageUrl = KFileDialog::getImageOpenUrl(currentUrl, this, i18n("Select Image"));
+  if (!imageUrl.isEmpty()) {
+    KWQCommandImage *kwqc = new KWQCommandImage(this, imageUrl);
+    m_undoStack->push(kwqc);
+  }
+}
+
+
+void KWQTableView::doVocabSound()
+{
+  KUrl currentUrl = model()->data(currentIndex(), KWQTableModel::SoundRole).toString();
+
+  KUrl soundUrl = KFileDialog::getOpenUrl(currentUrl, i18n("*|All Files"), this, i18n("Select Sound"));
+  if (!soundUrl.isEmpty()) {
+    KWQCommandSound *kwqc = new KWQCommandSound(this, soundUrl);
+    m_undoStack->push(kwqc);
+  }
+}
+
+
 void KWQTableView::doVocabShuffle()
 {
   setFocus();
@@ -660,6 +742,7 @@ void KWQTableView::doVocabShuffle()
   m_undoStack->push(kwqc);
 }
 
+
 void KWQTableView::updateKeyboardLayout()
 {
     QString layout;
@@ -673,4 +756,18 @@ void KWQTableView::updateKeyboardLayout()
     }
 }
 
+bool KWQTableView::selectionHasMoreThanText()
+{
+  bool result = false;
+  QModelIndexList indexes = selectionModel()->selectedIndexes();
+  foreach (const QModelIndex &idx, indexes) {
+     result = (!model()->data(idx, KWQTableModel::SoundRole).isNull() ||
+               !model()->data(idx, KWQTableModel::ImageRole).isNull());
+     if (result)
+       break;
+  }
+
+  return result;
+}
+
 #include "kwqtableview.moc"
