@@ -1,23 +1,27 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2008 Torsten Rahn <tackat@kde.org>"
+// Copyright 2006-2009 Torsten Rahn <tackat@kde.org>
 
 
 #include "GeoPainter.h"
+#include "GeoPainter_p.h"
 
-#include <QtCore/QDebug>
+#include <QtCore/QList>
 #include <QtGui/QPainterPath>
+#include <QtGui/QRegion>
 
 #include "AbstractProjection.h"
+#include "MarbleDebug.h"
 
 #include "GeoDataCoordinates.h"
 #include "GeoDataLineString.h"
 #include "GeoDataLinearRing.h"
+#include "GeoDataPoint.h"
 #include "GeoDataPolygon.h"
 
 #include "global.h"
@@ -25,122 +29,158 @@
 
 // #define MARBLE_DEBUG
 
+using namespace Marble;
 
-namespace Marble
-{
-
-class GeoPainterPrivate
-{
- public:
-    GeoPainterPrivate( ViewportParams *viewport, MapQuality mapQuality )
+GeoPainterPrivate::GeoPainterPrivate( ViewportParams *viewport, MapQuality mapQuality )
         : m_viewport( viewport ),
-	  m_mapQuality( mapQuality ),
-          m_x( new qreal[100] )
-    {
+        m_mapQuality( mapQuality ),
+        m_x( new qreal[100] )
+{
+}
+
+GeoPainterPrivate::~GeoPainterPrivate()
+{
+    delete[] m_x;
+}
+
+void GeoPainterPrivate::createPolygonsFromLineString( const GeoDataLineString & lineString,
+                                                      QVector<QPolygonF *> &polygons )
+{
+    AbstractProjection *projection = m_viewport->currentProjection();
+
+    projection->screenCoordinates( lineString, m_viewport, polygons );
+}
+
+void GeoPainterPrivate::createPolygonsFromLinearRing( const GeoDataLinearRing & linearRing,
+                                                      QVector<QPolygonF *> &polygons )
+{
+    AbstractProjection *projection = m_viewport->currentProjection();
+
+    projection->screenCoordinates( linearRing, m_viewport, polygons );
+}
+
+void GeoPainterPrivate::createAnnotationLayout (  qreal x, qreal y,
+                                                  QSizeF bubbleSize,
+                                                  qreal bubbleOffsetX, qreal bubbleOffsetY,
+                                                  qreal xRnd, qreal yRnd,
+                                                  QPainterPath& path, QRectF& rect )
+{
+    // TODO: MOVE this into an own Annotation class
+    qreal arrowPosition = 0.3;
+    qreal arrowWidth = 12.0;
+
+    qreal width =  bubbleSize.width();
+    qreal height = bubbleSize.height();
+
+    qreal dx =  ( bubbleOffsetX > 0 ) ? 1.0 : -1.0; // x-Mirror
+    qreal dy =  ( bubbleOffsetY < 0 ) ? 1.0 : -1.0; // y-Mirror
+
+    qreal x0 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd ) - xRnd *dx;
+    qreal x1 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd );
+    qreal x2 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd ) + xRnd * dx;
+    qreal x3 =  ( x + bubbleOffsetX ) - dx * arrowWidth / 2.0;
+    qreal x4 =  ( x + bubbleOffsetX ) + dx * arrowWidth / 2.0;
+    qreal x5 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd )- xRnd * dx;
+    qreal x6 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd );
+    qreal x7 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd ) + xRnd * dx;
+
+    qreal y0 =  ( y + bubbleOffsetY );
+    qreal y1 =  ( y + bubbleOffsetY ) - dy * yRnd;
+    qreal y2 =  ( y + bubbleOffsetY ) - dy * 2 * yRnd;
+    qreal y5 =  ( y + bubbleOffsetY ) - dy * ( height - 2 * yRnd );
+    qreal y6 =  ( y + bubbleOffsetY ) - dy * ( height - yRnd );
+    qreal y7 =  ( y + bubbleOffsetY ) - dy * height;
+
+    QPointF p1 ( x, y ); // pointing point
+    QPointF p2 ( x4, y0 );
+    QPointF p3 ( x6, y0 );
+    QPointF p4 ( x7, y1 );
+    QPointF p5 ( x7, y6 );
+    QPointF p6 ( x6, y7 );
+    QPointF p7 ( x1, y7 );
+    QPointF p8 ( x0, y6 );
+    QPointF p9 ( x0, y1 );
+    QPointF p10( x1, y0 );
+    QPointF p11( x3, y0 );
+
+    QRectF bubbleBoundingBox(  QPointF( x0, y7 ), QPointF( x7, y0 ) );
+
+    path.moveTo( p1 );
+    path.lineTo( p2 );
+
+    path.lineTo( p3 );
+    QRectF bottomRight( QPointF( x5, y2 ), QPointF( x7, y0 ) );
+    path.arcTo( bottomRight, 270.0, 90.0 );
+
+    path.lineTo( p5 );
+    QRectF topRight( QPointF( x5, y7 ), QPointF( x7, y5 ) );
+    path.arcTo( topRight, 0.0, 90.0 );
+
+    path.lineTo( p7 );
+    QRectF topLeft( QPointF( x0, y7 ), QPointF( x2, y5 ) );
+    path.arcTo( topLeft, 90.0, 90.0 );
+
+    path.lineTo( p9 );
+    QRectF bottomLeft( QPointF( x0, y2 ), QPointF( x2, y0 ) );
+    path.arcTo( bottomLeft, 180.0, 90.0 );
+
+    path.lineTo( p10 );
+    path.lineTo( p11 );
+    path.lineTo( p1 );
+
+    qreal left   = ( dx > 0 ) ? x1 : x6;
+    qreal right  = ( dx > 0 ) ? x6 : x1;
+    qreal top    = ( dy > 0 ) ? y6 : y1;
+    qreal bottom = ( dy > 0 ) ? y1 : y6;
+
+    rect.setTopLeft( QPointF( left, top ) );
+    rect.setBottomRight( QPointF( right, bottom ) );
+}
+
+GeoDataLinearRing GeoPainterPrivate::createLinearRingFromGeoRect( const GeoDataCoordinates & centerCoordinates,
+                                                                  qreal width, qreal height )
+{
+    qreal lon = 0.0;
+    qreal lat = 0.0;
+    qreal altitude = centerCoordinates.altitude();
+    centerCoordinates.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
+
+    lon = GeoDataCoordinates::normalizeLon( lon, GeoDataCoordinates::Degree );
+    lat = GeoDataCoordinates::normalizeLat( lat, GeoDataCoordinates::Degree );
+
+    qreal west = GeoDataCoordinates::normalizeLon( lon - width * 0.5, GeoDataCoordinates::Degree );
+    qreal east =  GeoDataCoordinates::normalizeLon( lon + width * 0.5, GeoDataCoordinates::Degree );
+
+    qreal north = GeoDataCoordinates::normalizeLat( lat + height * 0.5, GeoDataCoordinates::Degree );
+    qreal south = GeoDataCoordinates::normalizeLat( lat - height * 0.5, GeoDataCoordinates::Degree );
+
+    GeoDataCoordinates southWest( west, south,
+                                  altitude, GeoDataCoordinates::Degree );
+    GeoDataCoordinates southEast( east, south,
+                                  altitude, GeoDataCoordinates::Degree );
+    GeoDataCoordinates northEast( east, north,
+                                  altitude, GeoDataCoordinates::Degree );
+    GeoDataCoordinates northWest( west, north,
+                                  altitude, GeoDataCoordinates::Degree );
+
+    GeoDataLinearRing rectangle( Tessellate | RespectLatitudeCircle );
+
+    // If the width of the rect is larger as 180 degree, we have to enforce the long way.
+    if ( width >= 180 ) {
+        qreal center = lon;
+        GeoDataCoordinates southCenter( center, south, altitude, GeoDataCoordinates::Degree );
+        GeoDataCoordinates northCenter( center, north, altitude, GeoDataCoordinates::Degree );
+
+        rectangle << southWest << southCenter << southEast << northEast << northCenter << northWest;
+    }
+    else {
+        rectangle << southWest << southEast << northEast << northWest;
     }
 
-    ~GeoPainterPrivate()
-    {
-        delete[] m_x;
-    }
+    return rectangle;
+}
 
-    void createPolygonsFromLineString( const GeoDataLineString & lineString, 
-                   QVector<QPolygonF *> &polygons )
-    {
-        AbstractProjection *projection = m_viewport->currentProjection();
-
-        projection->screenCoordinates( lineString, m_viewport, polygons );
-    }
-
-
-    void createPolygonsFromLinearRing( const GeoDataLinearRing & linearRing, 
-                   QVector<QPolygonF *> &polygons )
-    {
-        AbstractProjection *projection = m_viewport->currentProjection();
-
-        projection->screenCoordinates( linearRing, m_viewport, polygons );
-    }
-
-
-    void createAnnotationLayout (  qreal x, qreal y, QSizeF bubbleSize, qreal bubbleOffsetX, qreal bubbleOffsetY, qreal xRnd, qreal yRnd, QPainterPath& path, QRectF& rect )
-    {
-        // TODO: MOVE this into an own Annotation class
-        qreal arrowPosition = 0.3;
-        qreal arrowWidth = 12.0;
-
-        qreal width =  bubbleSize.width();
-        qreal height = bubbleSize.height();
-
-        qreal dx =  ( bubbleOffsetX > 0 ) ? 1.0 : -1.0; // x-Mirror
-        qreal dy =  ( bubbleOffsetY < 0 ) ? 1.0 : -1.0; // y-Mirror
-
-        qreal x0 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd ) - xRnd *dx;
-        qreal x1 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd );
-        qreal x2 =  ( x + bubbleOffsetX ) - dx * ( 1.0 - arrowPosition ) * ( width - 2.0 * xRnd ) + xRnd * dx;
-        qreal x3 =  ( x + bubbleOffsetX ) - dx * arrowWidth / 2.0;
-        qreal x4 =  ( x + bubbleOffsetX ) + dx * arrowWidth / 2.0;
-        qreal x5 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd )- xRnd * dx;
-        qreal x6 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd );
-        qreal x7 =  ( x + bubbleOffsetX ) + dx * arrowPosition * ( width - 2.0 * xRnd ) + xRnd * dx;
-
-        qreal y0 =  ( y + bubbleOffsetY );
-        qreal y1 =  ( y + bubbleOffsetY ) - dy * yRnd;
-        qreal y2 =  ( y + bubbleOffsetY ) - dy * 2 * yRnd;
-        qreal y5 =  ( y + bubbleOffsetY ) - dy * ( height - 2 * yRnd );
-        qreal y6 =  ( y + bubbleOffsetY ) - dy * ( height - yRnd );
-        qreal y7 =  ( y + bubbleOffsetY ) - dy * height;
-
-        QPointF p1 ( x, y ); // pointing point
-        QPointF p2 ( x4, y0 );
-        QPointF p3 ( x6, y0 );
-        QPointF p4 ( x7, y1 );
-        QPointF p5 ( x7, y6 );
-        QPointF p6 ( x6, y7 );
-        QPointF p7 ( x1, y7 );
-        QPointF p8 ( x0, y6 );
-        QPointF p9 ( x0, y1 );
-        QPointF p10( x1, y0 );
-        QPointF p11( x3, y0 );
-
-        QRectF bubbleBoundingBox(  QPointF( x0, y7 ), QPointF( x7, y0 ) );
-
-        path.moveTo( p1 );
-        path.lineTo( p2 );
-
-        path.lineTo( p3 );
-        QRectF bottomRight( QPointF( x5, y2 ), QPointF( x7, y0 ) );
-        path.arcTo( bottomRight, 270.0, 90.0 );
-
-        path.lineTo( p5 );
-        QRectF topRight( QPointF( x5, y7 ), QPointF( x7, y5 ) );
-        path.arcTo( topRight, 0.0, 90.0 );
-
-        path.lineTo( p7 );
-        QRectF topLeft( QPointF( x0, y7 ), QPointF( x2, y5 ) );
-        path.arcTo( topLeft, 90.0, 90.0 );
-
-        path.lineTo( p9 );
-        QRectF bottomLeft( QPointF( x0, y2 ), QPointF( x2, y0 ) );
-        path.arcTo( bottomLeft, 180.0, 90.0 );
-
-        path.lineTo( p10 );
-        path.lineTo( p11 );
-        path.lineTo( p1 );
-
-        qreal left   = ( dx > 0 ) ? x1 : x6;
-        qreal right  = ( dx > 0 ) ? x6 : x1;
-        qreal top    = ( dy > 0 ) ? y6 : y1;
-        qreal bottom = ( dy > 0 ) ? y1 : y6;
-
-        rect.setTopLeft( QPointF( left, top ) );
-        rect.setBottomRight( QPointF( right, bottom ) );
-    }
-
-    ViewportParams  *m_viewport;
-    MapQuality       m_mapQuality;
-
-    qreal             *m_x;
-};
+// -------------------------------------------------------------------------------------------------
 
 GeoPainter::GeoPainter( QPaintDevice* pd, ViewportParams *viewport,
 			MapQuality mapQuality, bool clip )
@@ -149,29 +189,30 @@ GeoPainter::GeoPainter( QPaintDevice* pd, ViewportParams *viewport,
 {
 }
 
+
 GeoPainter::~GeoPainter()
 {
     delete d;
 }
 
-void GeoPainter::autoMapQuality ()
+
+void GeoPainter::autoMapQuality()
 {
-    bool antialiased = false;
-
-    if (   d->m_mapQuality == Marble::High
-        || d->m_mapQuality == Marble::Print ) {
-            antialiased = true;
-    }
-
+    const bool antialiased = d->m_mapQuality == HighQuality || d->m_mapQuality == PrintQuality;
     setRenderHint( QPainter::Antialiasing, antialiased );
 }
 
+
 MapQuality GeoPainter::mapQuality() const
 {
     return d->m_mapQuality;
 }
 
-void GeoPainter::drawAnnotation (  const GeoDataCoordinates & position, const QString & text, QSizeF bubbleSize, qreal bubbleOffsetX, qreal bubbleOffsetY, qreal xRnd, qreal yRnd )
+
+void GeoPainter::drawAnnotation( const GeoDataCoordinates & position,
+                                 const QString & text, QSizeF bubbleSize,
+                                 qreal bubbleOffsetX, qreal bubbleOffsetY,
+                                 qreal xRnd, qreal yRnd )
 {
     int pointRepeatNum;
     qreal y;
@@ -192,6 +233,7 @@ void GeoPainter::drawAnnotation (  const GeoDataCoordinates & position, const QS
     }
 }
 
+
 void GeoPainter::drawPoint (  const GeoDataCoordinates & position )
 {
     int pointRepeatNum;
@@ -209,15 +251,37 @@ void GeoPainter::drawPoint (  const GeoDataCoordinates & position )
     }
 }
 
-void GeoPainter::drawPoints (  const GeoDataCoordinates * points, int pointCount )
+
+QRegion GeoPainter::regionFromPoint ( const GeoDataCoordinates & position,
+                                      qreal width )
+{
+    return regionFromRect( position, width, width, false, 3 );
+}
+
+
+void GeoPainter::drawPoint( const GeoDataPoint & point )
+{
+    drawPoint( GeoDataCoordinates( point ) );
+}
+
+
+QRegion GeoPainter::regionFromPoint ( const GeoDataPoint & point,
+                                      qreal width )
+{
+    return regionFromRect( GeoDataCoordinates( point ), width, width, false, 3 );
+}
+
+
+void GeoPainter::drawPoints (  const GeoDataCoordinates * positions,
+                               int pointCount )
 {
     int pointRepeatNum;
     qreal y;
     bool globeHidesPoint;
     AbstractProjection *projection = d->m_viewport->currentProjection();
 
-    const GeoDataCoordinates * itPoint = points;
-    while( itPoint < points + pointCount ) {
+    const GeoDataCoordinates * itPoint = positions;
+    while( itPoint < positions + pointCount ) {
         bool visible = projection->screenCoordinates( *itPoint, d->m_viewport, d->m_x, y, pointRepeatNum, globeHidesPoint );
     
         if ( visible ) {
@@ -231,8 +295,15 @@ void GeoPainter::drawPoints (  const GeoDataCoordinates * points, int pointCount
     }
 }
 
-void GeoPainter::drawText ( const GeoDataCoordinates & position, const QString & text )
+
+void GeoPainter::drawText ( const GeoDataCoordinates & position,
+                            const QString & text )
 {
+    // Of course in theory we could have the "isGeoProjected" parameter used
+    // for drawText as well. However this would require us to convert all
+    // glyphs to PainterPaths / QPolygons. From QPolygons we could create
+    // GeoDataPolygons which could get painted on screen. Any patches appreciated ;-)
+
     int pointRepeatNum;
     qreal y;
     bool globeHidesPoint;
@@ -250,7 +321,10 @@ void GeoPainter::drawText ( const GeoDataCoordinates & position, const QString &
     }
 }
 
-void GeoPainter::drawEllipse ( const GeoDataCoordinates & centerPoint, qreal width, qreal height, bool isGeoProjected )
+
+void GeoPainter::drawEllipse ( const GeoDataCoordinates & centerPosition,
+                               qreal width, qreal height,
+                               bool isGeoProjected )
 {
     int pointRepeatNum;
     qreal y;
@@ -258,12 +332,13 @@ void GeoPainter::drawEllipse ( const GeoDataCoordinates & centerPoint, qreal wid
     AbstractProjection *projection = d->m_viewport->currentProjection();
 
     if ( !isGeoProjected ) {
-        bool visible = projection->screenCoordinates( centerPoint, d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
+        bool visible = projection->screenCoordinates( centerPosition, d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
 
         if ( visible ) {
             // Draw all the x-repeat-instances of the point on the screen
             for( int it = 0; it < pointRepeatNum; ++it ) {
-                QPainter::drawEllipse(  d->m_x[it] - width / 2.0, y - height / 2.0, width, height  );
+                QPainter::drawEllipse(  d->m_x[it] - width / 2.0,
+                                        y - height / 2.0, width, height  );
             }
         }
     }
@@ -271,8 +346,9 @@ void GeoPainter::drawEllipse ( const GeoDataCoordinates & centerPoint, qreal wid
         // Initialize variables
         qreal centerLon = 0.0;
         qreal centerLat = 0.0;
-        qreal altitude = centerPoint.altitude();
-        centerPoint.geoCoordinates( centerLon, centerLat, GeoDataCoordinates::Degree );
+        qreal altitude = centerPosition.altitude();
+        centerPosition.geoCoordinates( centerLon, centerLat,
+                                       GeoDataCoordinates::Degree );
 
         // Ensure a valid latitude range: 
         if ( centerLat + 0.5 * height > 90.0 || centerLat - 0.5 * height < -90.0 ) {
@@ -318,7 +394,11 @@ void GeoPainter::drawEllipse ( const GeoDataCoordinates & centerPoint, qreal wid
 
 }
 
-void GeoPainter::drawImage ( const GeoDataCoordinates & centerPoint, const QImage & image, bool isGeoProjected )
+
+QRegion GeoPainter::regionFromEllipse ( const GeoDataCoordinates & centerPosition,
+                                        qreal width, qreal height,
+                                        bool isGeoProjected,
+                                        qreal strokeWidth )
 {
     int pointRepeatNum;
     qreal y;
@@ -326,7 +406,87 @@ void GeoPainter::drawImage ( const GeoDataCoordinates & centerPoint, const QImag
     AbstractProjection *projection = d->m_viewport->currentProjection();
 
     if ( !isGeoProjected ) {
-        bool visible = projection->screenCoordinates( centerPoint, d->m_viewport, d->m_x, y, pointRepeatNum, image.size(), globeHidesPoint );
+        QRegion regions;
+
+        bool visible = projection->screenCoordinates( centerPosition, d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
+
+        if ( visible ) {
+            // Draw all the x-repeat-instances of the point on the screen
+            for( int it = 0; it < pointRepeatNum; ++it ) {
+                regions += QRegion( d->m_x[it] - width / 2.0,
+                                    y - height / 2.0,
+                                    width + strokeWidth,
+                                    height + strokeWidth,
+                                    QRegion::Ellipse );
+            }
+        }
+        return regions;
+    }
+    else {
+        // Initialize variables
+        qreal centerLon = 0.0;
+        qreal centerLat = 0.0;
+        qreal altitude = centerPosition.altitude();
+        centerPosition.geoCoordinates( centerLon, centerLat,
+                                       GeoDataCoordinates::Degree );
+
+        // Ensure a valid latitude range:
+        if ( centerLat + 0.5 * height > 90.0 || centerLat - 0.5 * height < -90.0 ) {
+            return QRegion();
+        }
+
+        // Don't show the ellipse if it's too small:
+        GeoDataLatLonBox ellipseBox( centerLat + 0.5 * height, centerLat - 0.5 * height,
+                                     centerLon + 0.5 * width,  centerLon - 0.5 * width,
+                                     GeoDataCoordinates::Degree );
+        if ( !d->m_viewport->viewLatLonAltBox().intersects( ellipseBox ) ||
+             !d->m_viewport->resolves( ellipseBox ) ) return QRegion();
+
+        GeoDataLinearRing ellipse;
+        qreal lon = 0.0;
+        qreal lat = 0.0;
+
+        // Optimizing the precision by determining the size which the
+        // ellipse covers on the screen:
+        qreal degreeResolution = d->m_viewport->angularResolution() * RAD2DEG;
+        // To create a circle shape even for very small precision we require uneven numbers:
+        int precision = width / degreeResolution / 8 + 1;
+        if ( precision > 81 ) precision = 81;
+
+        // Calculate the shape of the upper half of the ellipse:
+        for ( int i = 0; i <= precision; ++i ) {
+            qreal t = 1.0 - 2.0 * (qreal)(i) / (qreal)(precision);
+            lat = centerLat + 0.5 * height * sqrt( 1.0 - t * t );
+            lon = centerLon + 0.5 * width * t;
+            ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
+        }
+        // Calculate the shape of the lower half of the ellipse:
+        for ( int i = 0; i <= precision; ++i ) {
+            qreal t = 2.0 * (qreal)(i) / (qreal)(precision) -  1.0;
+            lat = centerLat - 0.5 * height * sqrt( 1.0 - t * t );
+            lon = centerLon + 0.5 * width * t;
+            ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
+        }
+
+        return regionFromPolygon( ellipse, Qt::OddEvenFill, strokeWidth );
+    }
+}
+    
+
+void GeoPainter::drawImage ( const GeoDataCoordinates & centerPosition,
+                             const QImage & image /*, bool isGeoProjected */ )
+{
+    // isGeoProjected = true would project the image/pixmap onto the globe. This
+    // requires to deal with the TextureMapping classes -> should get
+    // implemented later on
+
+    int pointRepeatNum;
+    qreal y;
+    bool globeHidesPoint;
+    AbstractProjection *projection = d->m_viewport->currentProjection();
+
+//    if ( !isGeoProjected ) {
+        bool visible = projection->screenCoordinates( centerPosition, d->m_viewport, d->m_x, y, pointRepeatNum, image.size(), globeHidesPoint );
 
         if ( visible ) {
             // Draw all the x-repeat-instances of the point on the screen
@@ -334,60 +494,78 @@ void GeoPainter::drawImage ( const GeoDataCoordinates & centerPoint, const QImag
                 QPainter::drawImage( d->m_x[it] - ( image.width() / 2 ), y - ( image.height() / 2 ), image );
             }
         }
-    }
+//    }
 }
 
-void GeoPainter::drawPixmap ( const GeoDataCoordinates & centerPoint, const QPixmap & pixmap, bool isGeoProjected )
+
+void GeoPainter::drawPixmap ( const GeoDataCoordinates & centerPosition,
+                              const QPixmap & pixmap /* , bool isGeoProjected */ )
 {
     int pointRepeatNum;
     qreal y;
     bool globeHidesPoint;
     AbstractProjection *projection = d->m_viewport->currentProjection();
 
-    if ( !isGeoProjected ) {
+//    if ( !isGeoProjected ) {
         // FIXME: Better visibility detection that takes the circle geometry into account
-        bool visible = projection->screenCoordinates( centerPoint, d->m_viewport, d->m_x, y, pointRepeatNum, pixmap.size(), globeHidesPoint );
+        bool visible = projection->screenCoordinates( centerPosition, d->m_viewport, d->m_x, y, pointRepeatNum, pixmap.size(), globeHidesPoint );
 
         if ( visible ) {
             // Draw all the x-repeat-instances of the point on the screen
             for( int it = 0; it < pointRepeatNum; ++it ) {
-                QPainter::drawPixmap( d->m_x[it] - ( pixmap.width() / 2 ), y - ( pixmap.height() / 2 ), pixmap );
+                QPainter::drawPixmap( d->m_x[it] - ( pixmap.width() / 2 ),
+                                      y - ( pixmap.height() / 2 ), pixmap );
             }
         }
-    }
+//    }
 }
 
-void GeoPainter::drawLine (  const GeoDataCoordinates & p1,  const GeoDataCoordinates & p2, bool isGeoProjected )
+
+void GeoPainter::drawLine (  const GeoDataCoordinates & startPoint,
+                             const GeoDataCoordinates & endPoint,
+                             bool isGeoProjected )
 {
     GeoDataLineString line;
     line.setTessellate( isGeoProjected );
 
-    GeoDataCoordinates c1 ( p1 );
-    GeoDataCoordinates c2 ( p2 );
-
-    line << c1 << c2;
+    line << startPoint << endPoint;
 
     drawPolyline( line ); 
 }
 
-void GeoPainter::drawPolyline ( const GeoDataLineString & lineString, const QString& labelText,
+
+QRegion GeoPainter::regionFromLine (const GeoDataCoordinates & startPoint,
+                                    const GeoDataCoordinates & endPoint,
+                                    bool isGeoProjected,
+                                    qreal strokeWidth )
+{
+    GeoDataLineString line;
+    line.setTessellate( isGeoProjected );
+
+    line << startPoint << endPoint;
+
+    return regionFromPolyline( line, strokeWidth );
+}
+
+
+void GeoPainter::drawPolyline ( const GeoDataLineString & lineString,
+                                const QString& labelText,
                                 LabelPositionFlags labelPositionFlags )
 {
-    // If the object is not visible in the viewport return 
+    // Immediately leave this method now if:
+    // - the object is not visible in the viewport or if
+    // - the size of the object is below the resolution of the viewport
     if ( ! d->m_viewport->viewLatLonAltBox().intersects( lineString.latLonAltBox() ) ||
-    // If the size of the object is below the resolution of the viewport then return
          ! d->m_viewport->resolves( lineString.latLonAltBox() )
         )
     {
-//        qDebug() << "LineString doesn't get displayed on the viewport";
+        // mDebug() << "LineString doesn't get displayed on the viewport";
         return;
     }
 
     QVector<QPolygonF*> polygons;
     d->createPolygonsFromLineString( lineString, polygons );
 
-//    qDebug() << "Number of polygons:" << polygons.count();
-
     if ( labelText.isEmpty() ) {
         foreach( QPolygonF* itPolygon, polygons ) {
             ClipPainter::drawPolyline( *itPolygon );
@@ -421,15 +599,52 @@ void GeoPainter::drawPolyline ( const GeoDataLineString & lineString, const QStr
     qDeleteAll( polygons );
 }
 
-void GeoPainter::drawPolygon ( const GeoDataLinearRing & linearRing, Qt::FillRule fillRule )
+
+QRegion GeoPainter::regionFromPolyline ( const GeoDataLineString & lineString,
+                                         qreal strokeWidth )
 {
-    // If the object is not visible in the viewport return
+    // Immediately leave this method now if:
+    // - the object is not visible in the viewport or if
+    // - the size of the object is below the resolution of the viewport
+    if ( ! d->m_viewport->viewLatLonAltBox().intersects( lineString.latLonAltBox() ) ||
+         ! d->m_viewport->resolves( lineString.latLonAltBox() )
+        )
+    {
+        // mDebug() << "LineString doesn't get displayed on the viewport";
+        return QRegion();
+    }
+
+    QList<QRegion> regions;
+    QPainterPath painterPath;
+
+    QVector<QPolygonF*> polygons;
+    d->createPolygonsFromLineString( lineString, polygons );
+
+    foreach( QPolygonF* itPolygon, polygons ) {
+        painterPath.addPolygon( *itPolygon );
+    }
+
+    qDeleteAll( polygons );
+
+    QPainterPathStroker stroker;
+    stroker.setWidth( strokeWidth );
+    QPainterPath strokePath = stroker.createStroke( painterPath );
+
+    return QRegion( strokePath.toFillPolygon().toPolygon(), Qt::WindingFill );
+}
+
+
+void GeoPainter::drawPolygon ( const GeoDataLinearRing & linearRing,
+                               Qt::FillRule fillRule )
+{
+    // Immediately leave this method now if:
+    // - the object is not visible in the viewport or if
+    // - the size of the object is below the resolution of the viewport
     if ( ! d->m_viewport->viewLatLonAltBox().intersects( linearRing.latLonAltBox() ) ||
-    // If the size of the object is below the resolution of the viewport then return
          ! d->m_viewport->resolves( linearRing.latLonAltBox() )
         )
     {
-//        qDebug() << "Polygon doesn't get displayed on the viewport";
+        // mDebug() << "Polygon doesn't get displayed on the viewport";
         return;
     }
 
@@ -472,7 +687,52 @@ void GeoPainter::drawPolygon ( const GeoDataLinearRing & linearRing, Qt::FillRul
     }
 }
 
-void GeoPainter::drawPolygon ( const GeoDataPolygon & polygon, Qt::FillRule fillRule )
+
+QRegion GeoPainter::regionFromPolygon ( const GeoDataLinearRing & linearRing,
+                                        Qt::FillRule fillRule, qreal strokeWidth )
+{
+    // Immediately leave this method now if:
+    // - the object is not visible in the viewport or if
+    // - the size of the object is below the resolution of the viewport
+    if ( ! d->m_viewport->viewLatLonAltBox().intersects( linearRing.latLonAltBox() ) ||
+         ! d->m_viewport->resolves( linearRing.latLonAltBox() )
+        )
+    {
+        return QRegion();
+    }
+
+    QRegion regions;
+
+    QVector<QPolygonF*> polygons;
+    d->createPolygonsFromLinearRing( linearRing, polygons );
+
+    if ( strokeWidth == 0 ) {
+        // This is the faster way
+        foreach( QPolygonF* itPolygon, polygons ) {
+            regions += QRegion ( (*itPolygon).toPolygon(), fillRule );
+        }
+    }
+    else {
+        QPainterPath painterPath;
+        foreach( QPolygonF* itPolygon, polygons ) {
+            painterPath.addPolygon( *itPolygon );
+        }
+
+        QPainterPathStroker stroker;
+        stroker.setWidth( strokeWidth );
+        QPainterPath strokePath = stroker.createStroke( painterPath );
+        painterPath = painterPath.united( strokePath );
+        regions = QRegion( painterPath.toFillPolygon().toPolygon() );
+    }
+
+    qDeleteAll( polygons );
+
+    return regions;
+}
+
+
+void GeoPainter::drawPolygon ( const GeoDataPolygon & polygon,
+                               Qt::FillRule fillRule )
 {
     // If the object is not visible in the viewport return 
     if ( ! d->m_viewport->viewLatLonAltBox().intersects( polygon.outerBoundary().latLonAltBox() ) ||
@@ -480,10 +740,10 @@ void GeoPainter::drawPolygon ( const GeoDataPolygon & polygon, Qt::FillRule fill
          ! d->m_viewport->resolves( polygon.outerBoundary().latLonAltBox() )
         )
     {
-//        qDebug() << "Polygon doesn't get displayed on the viewport";
+        // mDebug() << "Polygon doesn't get displayed on the viewport";
         return;
     }
-//    qDebug() << "Drawing Polygon";
+    // mDebug() << "Drawing Polygon";
 
     // Creating the outer screen polygons first
     QVector<QPolygonF*> outerPolygons;
@@ -514,7 +774,10 @@ void GeoPainter::drawPolygon ( const GeoDataPolygon & polygon, Qt::FillRule fill
     qDeleteAll( outerPolygons );    
 }
 
-void GeoPainter::drawRect ( const GeoDataCoordinates & centerCoordinates, qreal width, qreal height, bool isGeoProjected )
+
+void GeoPainter::drawRect ( const GeoDataCoordinates & centerCoordinates,
+                            qreal width, qreal height,
+                            bool isGeoProjected )
 {
     int pointRepeatNum;
     qreal y;
@@ -533,41 +796,52 @@ void GeoPainter::drawRect ( const GeoDataCoordinates & centerCoordinates, qreal
         }
     }
     else {
-        qreal lon = 0.0;
-        qreal lat = 0.0;
-        qreal altitude = centerCoordinates.altitude();
-        centerCoordinates.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
-
-        if ( width > 180.0 ) width = 180.0;
-        qreal lonLeft = lon - width * 0.5;
-        qreal lonRight =  lon + width * 0.5;
-
-        qreal latTop = lat + height * 0.5;
-        if ( latTop > +90.0 ) latTop = 90.0;
-        if ( latTop < -90.0 ) latTop = -90.0;
-
-        qreal latBottom = lat - height * 0.5;
-        if ( latBottom > +90.0 ) latBottom = 90.0;
-        if ( latBottom < -90.0 ) latBottom = -90.0;
-
-        GeoDataCoordinates bottomLeft( lonLeft, latBottom,
-                                    altitude, GeoDataCoordinates::Degree );
-        GeoDataCoordinates bottomRight( lonRight, latBottom,
-                                    altitude, GeoDataCoordinates::Degree );
-        GeoDataCoordinates topRight( lonRight, latTop,
-                                    altitude, GeoDataCoordinates::Degree );
-        GeoDataCoordinates topLeft( lonLeft, latTop,
-                                    altitude, GeoDataCoordinates::Degree );
-
-        GeoDataLinearRing rectangle( Tessellate | RespectLatitudeCircle );
-
-        rectangle << bottomLeft << bottomRight << topRight << topLeft;
-
-        drawPolygon( rectangle, Qt::OddEvenFill ); 
+        drawPolygon( d->createLinearRingFromGeoRect( centerCoordinates, width, height ),
+                     Qt::OddEvenFill );
     }
 }
 
-void GeoPainter::drawRoundRect ( const GeoDataCoordinates &centerPoint, int width, int height, int xRnd, int yRnd, bool isGeoProjected )
+
+QRegion GeoPainter::regionFromRect ( const GeoDataCoordinates & centerCoordinates,
+                                     qreal width, qreal height,
+                                     bool isGeoProjected,
+                                     qreal strokeWidth )
+{
+    int pointRepeatNum;
+    qreal y;
+    bool globeHidesPoint;
+    AbstractProjection *projection = d->m_viewport->currentProjection();
+
+
+    if ( !isGeoProjected ) {
+
+        QRegion regions;
+
+        bool visible = projection->screenCoordinates( centerCoordinates,
+                       d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
+
+        if ( visible ) {
+            // Draw all the x-repeat-instances of the point on the screen
+            for( int it = 0; it < pointRepeatNum; ++it ) {
+                regions += QRegion( d->m_x[it] - ( ( width + strokeWidth ) / 2.0 ),
+                                    y - ( ( height + strokeWidth ) / 2.0 ),
+                                    width + strokeWidth,
+                                    height + strokeWidth );
+            }
+        }
+        return regions;
+    }
+    else {
+        return regionFromPolygon( d->createLinearRingFromGeoRect( centerCoordinates, width, height ),
+                                  Qt::OddEvenFill, strokeWidth );
+    }
+}
+
+
+void GeoPainter::drawRoundRect ( const GeoDataCoordinates &centerPosition,
+                                 int width, int height,
+                                 int xRnd, int yRnd,
+                                 bool isGeoProjected )
 {
     int pointRepeatNum;
     qreal y;
@@ -576,7 +850,7 @@ void GeoPainter::drawRoundRect ( const GeoDataCoordinates &centerPoint, int widt
 
     if ( !isGeoProjected ) {
         // FIXME: Better visibility detection that takes the circle geometry into account
-        bool visible = projection->screenCoordinates( centerPoint, d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
+        bool visible = projection->screenCoordinates( centerPosition, d->m_viewport, d->m_x, y, pointRepeatNum, QSizeF( width, height ), globeHidesPoint );
 
         if ( visible ) {
             // Draw all the x-repeat-instances of the point on the screen
@@ -588,7 +862,8 @@ void GeoPainter::drawRoundRect ( const GeoDataCoordinates &centerPoint, int widt
 }
 
 
-    // Reenabling QPainter methods.
+
+// Reenabling QPainter methods.
 
 void GeoPainter::drawLine ( int x1, int y1, int x2, int y2 )
 {
@@ -615,11 +890,26 @@ void GeoPainter::drawEllipse ( const QRectF & rectangle )
     QPainter::drawEllipse( rectangle );
 }
 
-void GeoPainter::drawImage ( const QRect &target, const QImage &image, const QRect &source, Qt::ImageConversionFlags flags )
+void GeoPainter::drawImage ( const QRect &target,
+                             const QImage &image,
+                             const QRect &source,
+                             Qt::ImageConversionFlags flags )
 {
     QPainter::drawImage( target, image, source, flags );
 }
 
+void GeoPainter::drawImage( const QRect& rect,
+                            const QImage& image )
+{
+    QPainter::drawImage( rect, image );
+}
+
+void GeoPainter::drawImage( const QRectF& rect,
+                            const QImage& image )
+{
+    QPainter::drawImage( rect, image );
+}
+
 void GeoPainter::drawPixmap( int x, int y, const QPixmap &pixmap )
 {
     QPainter::drawPixmap( x, y, pixmap );
@@ -674,5 +964,3 @@ void GeoPainter::drawText ( const QPoint & position, const QString & text )
 {
     QPainter::drawText( position, text );
 }
-
-}
