@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -63,8 +63,7 @@ StackedTilePrivate::StackedTilePrivate( const TileId& id ) :
       m_resultTile(),
       m_byteCount(0),
       m_depth(0),
-      m_isGrayscale( false ),
-      m_forMergedLayerDecorator( false )
+      m_isGrayscale( false )
 {
 }
 
@@ -227,20 +226,17 @@ void StackedTilePrivate::calcByteCount()
 }
 
 
-StackedTile::StackedTile( TileId const& id, QObject * parent )
-    : AbstractTile( *new StackedTilePrivate( id ), parent ), d(0)
+StackedTile::StackedTile( TileId const& id, QVector<QSharedPointer<TextureTile> > const &tiles )
+    : AbstractTile( *new StackedTilePrivate( id ), 0 ), d(0)
 {
+    Q_ASSERT( !tiles.isEmpty() );
+
     // The d-ptr is cached as a member to avoid having to use d_func()
     // or the Q_D macro in the pixel() function. Otherwise it leads
     // to measurable runtime overhead because pixel() is called frequently.
     d = d_func();
-}
 
-StackedTile::StackedTile( StackedTilePrivate &dd, QObject * parent )
-    : AbstractTile( dd, parent ), d(0)
-{
-    // See comment above
-    d = d_func();
+    d->m_tiles = tiles;
 }
 
 StackedTile::~StackedTile()
@@ -257,38 +253,6 @@ bool StackedTile::isExpired() const
     return result;
 }
 
-bool StackedTile::forMergedLayerDecorator() const
-{
-    return d->m_forMergedLayerDecorator;
-}
-
-void StackedTile::setForMergedLayerDecorator()
-{
-    d->m_forMergedLayerDecorator = true;
-}
-
-void StackedTile::addTile( QSharedPointer<TextureTile> const & tile )
-{
-    d->m_tiles.append( tile );
-    deriveCompletionState();
-}
-
-void StackedTile::deriveCompletionState()
-{
-    QMap<TextureTile::State, int> count;
-    QVector<QSharedPointer<TextureTile> >::const_iterator pos = d->m_tiles.constBegin();
-    QVector<QSharedPointer<TextureTile> >::const_iterator const end = d->m_tiles.constEnd();
-    for (; pos != end; ++pos )
-        ++count[ (*pos)->state() ];
-
-    if ( count[ TextureTile::StateUptodate ] == d->m_tiles.size() )
-        d->m_state = TileComplete;
-    else if ( count[ TextureTile::StateEmpty ] == d->m_tiles.size() )
-        d->m_state = TileEmpty;
-    else
-        d->m_state = TilePartial;
-}
-
 void StackedTile::initJumpTables()
 {
     //    mDebug() << "Entered initJumpTables( bool ) of Tile" << d->m_id;
@@ -361,24 +325,18 @@ QImage * StackedTile::resultTile()
     return &d->m_resultTile;
 }
 
-bool StackedTile::hasTiles() const
-{
-    return !d->m_tiles.isEmpty();
-}
-
 void StackedTile::initResultTile()
 {
     // TODO: Free all the TextureTiles once the completion status is reached 
     // to save memory.
     
-    Q_ASSERT( hasTiles() );
+    Q_ASSERT( !d->m_tiles.isEmpty() );
     // if there are more than one active texture layers, we have to convert the
     // result tile into QImage::Format_ARGB32_Premultiplied to make blending possible
     const bool withConversion = d->m_tiles.count() > 1;
     QVector<QSharedPointer<TextureTile> >::const_iterator pos = d->m_tiles.constBegin();
     QVector<QSharedPointer<TextureTile> >::const_iterator const end = d->m_tiles.constEnd();
-    for (; pos != end; ++pos )
-        if ( (*pos)->state() != TextureTile::StateEmpty ) {
+    for (; pos != end; ++pos ) {
             Blending const * const blending = (*pos)->blending();
             if ( blending ) {
                 mDebug() << "StackedTile::initResultTile: blending";
@@ -389,7 +347,7 @@ void StackedTile::initResultTile()
                     "no blending defined => copying top over bottom image";
                 d->setResultTile( *pos, withConversion );
             }
-        }
+    }
 
     initJumpTables();
 
