@@ -20,7 +20,7 @@
 */
 #include "osm-namefinder/NamedTagHandler.h"
 
-#include <QtCore/QDebug>
+#include "MarbleDebug.h"
 
 #include "GeoDataDocument.h"
 #include "GeoDataPlacemark.h"
@@ -34,20 +34,20 @@ namespace OsmNamefinder
 {
 
 static GeoTagHandlerRegistrar
-handler( GeoTagHandler::QualifiedName( tag_named, tag_namespace ),
+namedTagHandler( GeoTagHandler::QualifiedName( tag_named, tag_namespace ),
          new NamedTagHandler );
 
 
 GeoNode * NamedTagHandler::parse( GeoParser & parser ) const
 {
     Q_ASSERT( parser.isStartElement() && parser.isValidElement( tag_named ));
-    qDebug() << "NamedTagHandler";
+    mDebug() << "NamedTagHandler";
 
     GeoStackItem parentItem = parser.parentElement();
-    qDebug() << "parentItem:" << parentItem.qualifiedName().first;
+    mDebug() << "parentItem:" << parentItem.qualifiedName().first;
 
     // FIXME: better check tags before?
-    GeoDataPlacemark named;
+    GeoDataPlacemark *named = new GeoDataPlacemark;
 
     // FIXME: attribute type
 //     const ItemType type = getItemType( parser.attribute( attr_type ).trimmed() );
@@ -58,7 +58,7 @@ GeoNode * NamedTagHandler::parse( GeoParser & parser ) const
     const QString idStr = parser.attribute( attr_id ).trimmed();
     if ( !idStr.isEmpty() )
         id = idStr.toInt();
-    named.setId( id );
+    named->setId( id );
 
     // attributes lat and lon
     qreal lat = 0.0;
@@ -70,11 +70,11 @@ GeoNode * NamedTagHandler::parse( GeoParser & parser ) const
     const QString lonStr = parser.attribute( attr_lon ).trimmed();
     if ( !lonStr.isEmpty() )
         lon = lonStr.toDouble();
-    named.setCoordinate( GeoDataPoint( lon, lat, 0.0, GeoDataPoint::Degree, 0 ));
+    named->setCoordinate( GeoDataPoint( lon, lat, 0.0, GeoDataPoint::Degree, 0 ));
 
     // attribute name
     const QString name = parser.attribute( attr_name ).trimmed();
-    named.setName( name );
+    named->setName( name );
 
     // attribute rank
     int rank = 0;
@@ -89,21 +89,21 @@ GeoNode * NamedTagHandler::parse( GeoParser & parser ) const
     const QString zoomStr = parser.attribute( attr_zoom ).trimmed();
     if ( !zoomStr.isEmpty() )
         suggestedZoomLevel = zoomStr.toInt();
-    named.setPopularityIndex( suggestedZoomLevel );
+    named->setPopularityIndex( suggestedZoomLevel );
 
-    qDebug() << "parsed named:" << named.name() << " lon:" << lon << " lat:" << lat;
+    mDebug() << "parsed named:" << named->name() << " lon:" << lon << " lat:" << lat;
 
     if ( parentItem.represents( tag_searchresults )) {
-        qDebug() << "added via parent searchresults";
+        mDebug() << "added via parent searchresults";
         GeoDataContainer * const parent = parentItem.nodeAs<GeoDataContainer>();
         Q_ASSERT( parent );
         parent->append( named );
 
     } else if ( parentItem.represents( tag_place )) {
-        qDebug() << "added via parent place, not implemented";
+        mDebug() << "added via parent place, not implemented";
 
     } else if ( parentItem.represents( tag_nearestplaces )) {
-        qDebug() << "added via parent nearestplaces, not implemented";
+        mDebug() << "added via parent nearestplaces, not implemented";
 
     } else if ( parentItem.first.first.isEmpty() && !parentItem.second ) {
         // ok, it is the parentItem would be searchresults, but we
