@@ -19,8 +19,8 @@
     Boston, MA 02110-1301, USA.
 */
 
-#ifndef GEOSCENESECTION_H
-#define GEOSCENESECTION_H
+#ifndef MARBLE_GEOSCENESECTION_H
+#define MARBLE_GEOSCENESECTION_H
 
 #include <QtCore/QString>
 #include <QtCore/QVector>
@@ -64,7 +64,9 @@ class GeoSceneSection : public GeoNode
     int  spacing() const;
     void setSpacing( int spacing );
 
- protected:
+ private:
+    Q_DISABLE_COPY( GeoSceneSection )
+
     /// The vector holding all the items in the legend section.
     /// (We want to preserve the order and don't care 
     /// much about speed here), so we don't use a hash
@@ -76,11 +78,8 @@ class GeoSceneSection : public GeoNode
 
     bool    m_checkable;
     int     m_spacing;
-
- private:
-    Q_DISABLE_COPY( GeoSceneSection )
 };
 
 }
 
-#endif // GEOSCENESECTION_H
+#endif
