@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -16,6 +16,7 @@
 #include <math.h>
 
 #include <QtCore/QString>
+#include <QtGui/QColor>
 
 #include "marble_export.h"
 
@@ -203,22 +204,46 @@ const qreal FT2M = 1.0 / M2FT;
 const qreal DEG2RAD = M_PI / 180.0;
 const qreal RAD2DEG = 180.0 / M_PI;
 
-// Converstion meter vs kilometer
+// Conversion meter vs kilometer
 const qreal KM2METER = 1000.0;
 const qreal METER2KM = 1.0 / KM2METER;
 
+//Conversion hour vs minute
+const qreal HOUR2MIN = 60.0;
+const qreal MIN2HOUR = 1.0 / HOUR2MIN;
+
+//Conversion (time) minute vs second
+const qreal MIN2SEC = 60.0;
+const qreal SEC2MIN = 1.0 / MIN2SEC;
+
+//Conversion hour vs second
+const qreal HOUR2SEC = 3600.0;
+const qreal SEC2HOUR = 1.0 / HOUR2SEC;
+
+// Colors taken from the oxygen palette
+// When extending, please use the names and rgb values as defined in
+// http://websvn.kde.org/*checkout*/trunk/playground/artwork/Oxygen/utils/oxygen.gpl
+QColor const oxygenBrownOrange4 = QColor::fromRgb( 191, 94, 0 );
+QColor const oxygenAluminumGray4 = QColor::fromRgb( 136, 138, 133 );
+QColor const oxygenSkyBlue4 = QColor::fromRgb( 0, 87, 174 );
+QColor const oxygenSunYellow6 = QColor::fromRgb( 227, 173, 0 );
+QColor const oxygenSeaBlue2 = QColor::fromRgb( 0, 196, 204 );
+QColor const oxygenHotOrange4 = QColor::fromRgb( 236, 115, 49 );
+QColor const oxygenBrickRed4 = QColor::fromRgb( 226, 8, 0 );
+QColor const oxygenForestGreen4 = QColor::fromRgb( 55, 164, 44 );
+
 // Version definitions to use with an external application (as digiKam)
 
 // String for about dialog and http user agent
 // FIXME: check if blanks are allowed in user agent version numbers
-const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.10.3" );
+const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.11.0 (Stable Release)" );
 
 // API Version id:
 // form : 0xMMmmpp
 //        MM = major revision.
 //        mm = minor revision.
 //        pp = patch revision.
-#define MARBLE_VERSION 0x000a03
+#define MARBLE_VERSION 0x000b00
 
 static const char NOT_AVAILABLE[] = QT_TR_NOOP("not available");
 
@@ -234,6 +259,15 @@ const int maxBaseTileLevel = 4;
 class MarbleGlobalPrivate;
 class MarbleLocale;
 
+#ifdef __GNUC__
+#define MARBLE_DEPRECATED(func) func __attribute__ ((deprecated))
+#elif defined(_MSC_VER)
+#define MARBLE_DEPRECATED(func) __declspec(deprecated) func
+#else
+#pragma message("WARNING: You need to implement MARBLE_DEPRECATED for this compiler in global.h")
+#define MARBLE_DEPRECATED(func) func
+#endif
+
 class  MARBLE_EXPORT MarbleGlobal
 {
  public:
@@ -244,7 +278,8 @@ class  MARBLE_EXPORT MarbleGlobal
     
     enum Profile {
         Default = 0x0,
-        SmallScreen = 0x1
+        SmallScreen = 0x1,
+        HighResolution = 0x2
     };
     
     Q_DECLARE_FLAGS( Profiles, Profile )
