@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -19,7 +19,10 @@
 #include "MarbleWidget.h"
 #include "MarbleWidgetPopupMenu.h"
 #include "RoutingModel.h"
-#include "RouteSkeleton.h"
+#include "RouteRequest.h"
+#include "MarbleModel.h"
+#include "AlternativeRoutesModel.h"
+#include "RoutingManager.h"
 
 #include <QtCore/QMap>
 #include <QtGui/QAbstractProxyModel>
@@ -29,6 +32,8 @@
 #include <QtGui/QMenu>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPixmap>
+#include <QtGui/QComboBox>
+#include <QtGui/QFileDialog>
 
 namespace Marble
 {
@@ -48,14 +53,16 @@ class RoutingLayerPrivate
     };
 
     typedef PaintRegion<QModelIndex> ModelRegion;
-    typedef PaintRegion<int> SkeletonRegion;
+    typedef PaintRegion<int> RequestRegion;
 
 public:
     RoutingLayer *q;
 
     QList<ModelRegion> m_instructionRegions;
 
-    QList<SkeletonRegion> m_regions;
+    QList<RequestRegion> m_regions;
+
+    QList<RequestRegion> m_alternativeRouteRegions;
 
     QList<ModelRegion> m_placemarks;
 
@@ -89,7 +96,7 @@ public:
 
     QSize m_pixmapSize;
 
-    RouteSkeleton *m_routeSkeleton;
+    RouteRequest *m_routeRequest;
 
     MarbleWidgetPopupMenu *m_contextMenu;
 
@@ -97,6 +104,10 @@ public:
 
     int m_activeMenuIndex;
 
+    QComboBox *m_alternativeRoutesView;
+
+    AlternativeRoutesModel* m_alternativeRoutesModel;
+
     /** Constructor */
     explicit RoutingLayerPrivate( RoutingLayer *parent, MarbleWidget *widget );
 
@@ -110,6 +121,9 @@ public:
     // Inlined to avoid the function call overhead. Having functions here is just to
     // keep the code clean
 
+    /** Returns the same color as the given one with its alpha channel adjusted to the given value */
+    inline QColor alphaAdjusted( const QColor &color, int alpha ) const;
+
     /**
       * Returns the start or destination position if Ctrl key is among the
       * provided modifiers, the cached insert position otherwise
@@ -122,8 +136,10 @@ public:
     /** Paint waypoint polygon */
     inline void renderRoute( GeoPainter *painter );
 
+    inline void renderAlternativeRoutes( GeoPainter *painter );
+
     /** Paint icons for trip points etc */
-    inline void renderSkeleton( GeoPainter *painter );
+    inline void renderRequest( GeoPainter *painter );
 
     /** Insert via points or emit position signal, if appropriate */
     inline bool handleMouseButtonRelease( QMouseEvent *e );
@@ -140,6 +156,9 @@ public:
     /** True if the given point (screen coordinates) is among the route instruction points */
     inline bool isInfoPoint( const QPoint &point );
 
+    /** True if the given point (screen coordinates) is above an alternative route */
+    inline bool isAlternativeRoutePoint( const QPoint &point );
+
     /** Paint the stopover indicator pixmap at the given position. Also repaints the old position */
     inline void paintStopOver( QRect position );
 
@@ -151,15 +170,21 @@ RoutingLayerPrivate::RoutingLayerPrivate( RoutingLayer *parent, MarbleWidget *wi
         q( parent ), m_proxyModel( 0 ), m_movingIndex( -1 ), m_marbleWidget( widget ),
         m_targetPixmap( ":/data/bitmaps/routing_pick.png" ), m_dragStopOverRightIndex( -1 ),
         m_pointSelection( false ), m_routingModel( 0 ), m_placemarkModel( 0 ), m_selectionModel( 0 ),
-        m_routeDirty( false ), m_pixmapSize( 22, 22 ), m_routeSkeleton( 0 ), m_activeMenuIndex( -1 )
+        m_routeDirty( false ), m_pixmapSize( 22, 22 ), m_routeRequest( 0 ), m_activeMenuIndex( -1 ),
+        m_alternativeRoutesView( 0 ),
+        m_alternativeRoutesModel( widget->model()->routingManager()->alternativeRoutesModel() )
 {
     m_contextMenu = new MarbleWidgetPopupMenu( m_marbleWidget, m_marbleWidget->model() );
     m_removeViaPointAction = new QAction( QObject::tr( "&Remove this destination" ), q );
     QObject::connect( m_removeViaPointAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
     m_contextMenu->addAction( Qt::RightButton, m_removeViaPointAction );
     QAction *exportAction = new QAction( QObject::tr( "&Export route..." ), q );
-    QObject::connect( exportAction, SIGNAL( triggered() ), q, SIGNAL( exportRequested() ) );
+    QObject::connect( exportAction, SIGNAL( triggered() ), q, SLOT( exportRoute() ) );
     m_contextMenu->addAction( Qt::RightButton, exportAction );
+    if ( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        m_pixmapSize = QSize( 38, 38 );
+    }
+
 }
 
 void RoutingLayerPrivate::showContextMenu( const QPoint &pos )
@@ -170,11 +195,11 @@ void RoutingLayerPrivate::showContextMenu( const QPoint &pos )
 int RoutingLayerPrivate::viaInsertPosition( Qt::KeyboardModifiers modifiers ) const
 {
     if ( modifiers & Qt::ControlModifier ) {
-        bool leftHand = m_routeSkeleton->size() / 2 >= m_dragStopOverRightIndex;
-        if ( leftHand && m_routeSkeleton->size() > 2 ) {
+        bool leftHand = m_routeRequest->size() / 2 >= m_dragStopOverRightIndex;
+        if ( leftHand && m_routeRequest->size() > 2 ) {
             return 0;
         } else {
-            return m_routeSkeleton->size();
+            return m_routeRequest->size();
         }
     } else {
         return m_dragStopOverRightIndex;
@@ -206,6 +231,25 @@ void RoutingLayerPrivate::renderPlacemarks( GeoPainter *painter )
     }
 }
 
+void RoutingLayerPrivate::renderAlternativeRoutes( GeoPainter *painter )
+{
+    m_alternativeRouteRegions.clear();
+
+    QPen grayPen( alphaAdjusted( oxygenAluminumGray4, 200 ) );
+    grayPen.setWidth( 5 );
+    painter->setPen( grayPen );
+
+    for ( int i=0; i<m_alternativeRoutesModel->rowCount(); ++i ) {
+        GeoDataDocument* route = m_alternativeRoutesModel->route( i );
+        if ( route ) {
+            GeoDataLineString* points = AlternativeRoutesModel::waypoints( route );
+            painter->drawPolyline( *points );
+            QRegion region = painter->regionFromPolyline( *points, 8 );
+            m_alternativeRouteRegions.push_back( RequestRegion( i, region ) );
+        }
+    }
+}
+
 void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
 {
     m_instructionRegions.clear();
@@ -221,7 +265,7 @@ void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
         }
     }
 
-    QPen bluePen( QColor::fromRgb( 0, 87, 174, 200 ) ); // blue, oxygen palette
+    QPen bluePen( alphaAdjusted( oxygenSkyBlue4, 200 ) );
     bluePen.setWidth( 5 );
     if ( m_routeDirty ) {
         bluePen.setStyle( Qt::DotLine );
@@ -233,7 +277,7 @@ void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
 
     bluePen.setWidth( 2 );
     painter->setPen( bluePen );
-    painter->setBrush( QBrush( QColor::fromRgb( 136, 138, 133, 200 ) ) ); // gray, oxygen palette
+    painter->setBrush( QBrush( alphaAdjusted( oxygenAluminumGray4, 200 ) ) );
 
     if ( !m_dropStopOver.isNull() ) {
         int dx = 1 + m_pixmapSize.width() / 2;
@@ -241,7 +285,7 @@ void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
         QPoint center = m_dropStopOver - QPoint( dx, dy );
         painter->drawPixmap( center, m_targetPixmap );
 
-        if ( !m_dragStopOver.isNull() && m_dragStopOverRightIndex >= 0 && m_dragStopOverRightIndex <= m_routeSkeleton->size() ) {
+        if ( !m_dragStopOver.isNull() && m_dragStopOverRightIndex >= 0 && m_dragStopOverRightIndex <= m_routeRequest->size() ) {
             QPoint moved = m_dropStopOver - m_dragStopOver;
             if ( moved.manhattanLength() > 10 ) {
                 qreal lon( 0.0 ), lat( 0.0 );
@@ -251,10 +295,10 @@ void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
                     bluePen.setStyle( Qt::DotLine );
                     painter->setPen( bluePen );
                     if ( m_dragStopOverRightIndex > 0 ) {
-                        painter->drawLine( drag, m_routeSkeleton->at( m_dragStopOverRightIndex-1 ) );
+                        painter->drawLine( drag, m_routeRequest->at( m_dragStopOverRightIndex-1 ) );
                     }
-                    if ( m_dragStopOverRightIndex < m_routeSkeleton->size() ) {
-                        painter->drawLine( drag, m_routeSkeleton->at( m_dragStopOverRightIndex ) );
+                    if ( m_dragStopOverRightIndex < m_routeRequest->size() ) {
+                        painter->drawLine( drag, m_routeRequest->at( m_dragStopOverRightIndex ) );
                     }
                     bluePen.setStyle( Qt::SolidLine );
                     painter->setPen( bluePen );
@@ -268,39 +312,67 @@ void RoutingLayerPrivate::renderRoute( GeoPainter *painter )
         GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( RoutingModel::CoordinateRole ) );
         RoutingModel::RoutingItemType type = qVariantValue<RoutingModel::RoutingItemType>( index.data( RoutingModel::TypeRole ) );
 
-        if ( type == RoutingModel::Instruction ) {
-            painter->setBrush( QBrush( QColor::fromRgb( 136, 138, 133, 200 ) ) ); // gray, oxygen palette
+        if ( type == RoutingModel::Instruction && m_proxyModel && m_selectionModel ) {
+
+            painter->setBrush( QBrush( alphaAdjusted( oxygenAluminumGray4, 200 ) ) );
             QModelIndex proxyIndex = m_proxyModel->mapFromSource( index );
             if ( m_selectionModel->selection().contains( proxyIndex ) ) {
                 painter->setPen( QColor( Qt::black ) );
-                painter->setBrush( QBrush( QColor::fromRgb( 227, 173, 0, 200 ) ) ); // yellow, oxygen palette
+                painter->setBrush( QBrush( oxygenSunYellow6 ) );
                 painter->drawAnnotation( pos, index.data().toString(), QSize( 120, 60 ), 10, 30, 15, 15 );
 
+                GeoDataLineString currentRoutePoints = qVariantValue<GeoDataLineString>( index.data( RoutingModel::InstructionWayPointRole ) );
+
+                QPen brightBluePen( alphaAdjusted( oxygenSeaBlue2, 200 ) );
+
+                brightBluePen.setWidth( 6 );
+                if ( m_routeDirty ) {
+                    brightBluePen.setStyle( Qt::DotLine );
+                }
+                painter->setPen( brightBluePen );
+
+                painter->drawPolyline( currentRoutePoints );
+
                 painter->setPen( bluePen );
-                painter->setBrush( QBrush( QColor::fromRgb( 236, 115, 49, 200 ) ) ); // orange, oxygen palette
+                painter->setBrush( QBrush( alphaAdjusted( oxygenHotOrange4, 200 ) ) );
             }
 
             QRegion region = painter->regionFromEllipse( pos, 12, 12 );
             m_instructionRegions.push_front( ModelRegion( index, region ) );
             painter->drawEllipse( pos, 8, 8 );
+
         } else if ( !m_routeDirty && type == RoutingModel::Error ) {
             painter->setPen( QColor( Qt::white ) );
-            painter->setBrush( QBrush( QColor::fromRgb( 226, 8, 0, 200 ) ) ); // red, oxygen palette
-            painter->drawAnnotation( pos, index.data().toString(), QSize( 240, 100 ), 10, 30, 15, 15 );
+            painter->setBrush( QBrush( alphaAdjusted( oxygenBrickRed4, 200 ) ) );
+            painter->drawAnnotation( pos, index.data().toString(), QSize( 180, 80 ), 10, 30, 15, 15 );
+        }
+
+        if( !m_routingModel->deviatedFromRoute() ) {
+            qreal timeRemaining = m_routingModel->remainingTime();
+            qreal thresholdMinimum = 3.0;
+            GeoDataCoordinates location = m_routingModel->instructionPoint();
+            QString nextInstruction = m_routingModel->instructionText();
+            if( timeRemaining < thresholdMinimum && !nextInstruction.isEmpty() ) {
+                painter->setBrush( QBrush( oxygenBrownOrange4 ) );
+                painter->drawEllipse( location, 20, 20 );
+            } else {
+                painter->setBrush( QBrush( oxygenSunYellow6 ) );
+                painter->drawEllipse( location, 12, 12 );
+            }
         }
     }
 }
 
-void RoutingLayerPrivate::renderSkeleton( GeoPainter *painter )
+void RoutingLayerPrivate::renderRequest( GeoPainter *painter )
 {
     m_regions.clear();
-    for ( int i = 0; i < m_routeSkeleton->size(); ++i ) {
-        GeoDataCoordinates pos = m_routeSkeleton->at( i );
+    for ( int i = 0; i < m_routeRequest->size(); ++i ) {
+        GeoDataCoordinates pos = m_routeRequest->at( i );
         if ( pos.longitude() != 0.0 && pos.latitude() != 0.0 ) {
-            QPixmap pixmap = m_routeSkeleton->pixmap( i );
+            QPixmap pixmap = m_routeRequest->pixmap( i );
             painter->drawPixmap( pos, pixmap );
             QRegion region = painter->regionFromRect( pos, pixmap.width(), pixmap.height() );
-            m_regions.push_front( SkeletonRegion( i, region ) );
+            m_regions.push_front( RequestRegion( i, region ) );
         }
     }
 }
@@ -311,20 +383,27 @@ void RoutingLayerPrivate::storeDragPosition( const QPoint &pos )
     m_dragStopOverRightIndex = -1;
 
     qreal lon( 0.0 ), lat( 0.0 );
-    if ( m_routeSkeleton && !pos.isNull()
+    if ( m_routeRequest && !pos.isNull()
         && m_marbleWidget->geoCoordinates( pos.x(), pos.y(), lon, lat, GeoDataCoordinates::Radian ) ) {
         GeoDataCoordinates waypoint( lon, lat );
-        m_dragStopOverRightIndex = m_routingModel->rightNeighbor( waypoint, m_routeSkeleton );
+        m_dragStopOverRightIndex = m_routingModel->rightNeighbor( waypoint, m_routeRequest );
     }
 }
 
+QColor RoutingLayerPrivate::alphaAdjusted( const QColor &color, int alpha ) const
+{
+    QColor result( color );
+    result.setAlpha( alpha );
+    return result;
+}
+
 bool RoutingLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
 {
     if ( m_pointSelection ) {
         return e->button() == Qt::LeftButton;
     }
 
-    foreach( const SkeletonRegion &region, m_regions ) {
+    foreach( const RequestRegion &region, m_regions ) {
         if ( region.region.contains( e->pos() ) ) {
             if ( e->button() == Qt::LeftButton ) {
                 m_movingIndex = region.index;
@@ -342,8 +421,8 @@ bool RoutingLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
     }
 
     foreach( const ModelRegion &region, m_instructionRegions ) {
-        if ( region.region.contains( e->pos() ) ) {
-            if ( e->button() == Qt::LeftButton ) {
+        if ( region.region.contains( e->pos() ) && m_selectionModel ) {
+            if ( e->button() == Qt::LeftButton && m_proxyModel ) {
                 QModelIndex index = m_proxyModel->mapFromSource( region.index );
                 QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
                 if ( m_selectionModel->isSelected( index ) ) {
@@ -382,6 +461,13 @@ bool RoutingLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
         return false;
     }
 
+    foreach( const RequestRegion &region, m_alternativeRouteRegions ) {
+        if ( region.region.contains( e->pos() ) ) {
+            m_alternativeRoutesView->setCurrentIndex( region.index );
+            return true;
+        }
+    }
+
     foreach( const ModelRegion &region, m_placemarks ) {
         if ( region.region.contains( e->pos() ) ) {
             emit q->placemarkSelected( region.index );
@@ -425,11 +511,11 @@ bool RoutingLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
         }
 
         qreal lon( 0.0 ), lat( 0.0 );
-        if ( m_dragStopOverRightIndex >= 0 && m_dragStopOverRightIndex <= m_routeSkeleton->size()
+        if ( m_dragStopOverRightIndex >= 0 && m_dragStopOverRightIndex <= m_routeRequest->size()
                 && m_marbleWidget->geoCoordinates( m_dropStopOver.x(), m_dropStopOver.y(), lon, lat, GeoDataCoordinates::Radian ) ) {
             GeoDataCoordinates position( lon, lat );
             m_dragStopOverRightIndex = viaInsertPosition( e->modifiers() );
-            m_routeSkeleton->insert( m_dragStopOverRightIndex, position );
+            m_routeRequest->insert( m_dragStopOverRightIndex, position );
             clearStopOver();
             emit q->routeDirty();
             return true;
@@ -456,14 +542,14 @@ bool RoutingLayerPrivate::handleMouseMove( QMouseEvent *e )
 
         if ( m_movingIndex >= 0 ) {
             GeoDataCoordinates moved( lon, lat );
-            m_routeSkeleton->setPosition( m_movingIndex, moved );
+            m_routeRequest->setPosition( m_movingIndex, moved );
             m_marbleWidget->setCursor( Qt::ArrowCursor );
         } else if ( !m_dragStopOver.isNull() ) {
             // Repaint only that region of the map that is affected by the change
             m_dragStopOverRightIndex = viaInsertPosition( e->modifiers() );
             QRect dirty = m_routeRegion.boundingRect();
-            dirty |= QRect( m_dropStopOver, QSize( 22, 22 ) );
-            dirty |= QRect( e->pos(), QSize( 22, 22 ) );
+            dirty |= QRect( m_dropStopOver, m_pixmapSize );
+            dirty |= QRect( e->pos(), m_pixmapSize );
             if ( e->buttons() & Qt::LeftButton ) {
                 m_dropStopOver = e->pos();
             } else {
@@ -480,7 +566,10 @@ bool RoutingLayerPrivate::handleMouseMove( QMouseEvent *e )
             m_marbleWidget->setCursor( Qt::ArrowCursor );
         } else if ( !m_dropStopOver.isNull() ) {
             clearStopOver();
-        } else {
+        } else if ( isAlternativeRoutePoint( e->pos() ) ) {
+            m_marbleWidget->setCursor( Qt::ArrowCursor );
+        }
+        else {
             return false;
         }
 
@@ -505,7 +594,7 @@ bool RoutingLayerPrivate::handleKeyEvent( QKeyEvent *e )
 
 bool RoutingLayerPrivate::isInfoPoint( const QPoint &point )
 {
-    foreach( const SkeletonRegion &region, m_regions ) {
+    foreach( const RequestRegion &region, m_regions ) {
         if ( region.region.contains( point ) ) {
             return true;
         }
@@ -520,6 +609,17 @@ bool RoutingLayerPrivate::isInfoPoint( const QPoint &point )
     return false;
 }
 
+ bool RoutingLayerPrivate::isAlternativeRoutePoint( const QPoint &point )
+ {
+     foreach( const RequestRegion &region, m_alternativeRouteRegions ) {
+         if ( region.region.contains( point ) ) {
+             return true;
+         }
+     }
+
+     return false;
+ }
+
 void RoutingLayerPrivate::paintStopOver( QRect dirty )
 {
     m_marbleWidget->repaint( m_dirtyRect );
@@ -541,6 +641,8 @@ RoutingLayer::RoutingLayer( MarbleWidget *widget, QWidget *parent ) :
         QObject( parent ), d( new RoutingLayerPrivate( this, widget ) )
 {
     widget->installEventFilter( this );
+    connect( widget->model()->routingManager(), SIGNAL( stateChanged( RoutingManager::State, RouteRequest* ) ),
+             this, SLOT( updateRouteState( RoutingManager::State, RouteRequest* ) ) );
 }
 
 RoutingLayer::~RoutingLayer()
@@ -571,12 +673,16 @@ bool RoutingLayer::render( GeoPainter *painter, ViewportParams *viewport,
         d->renderPlacemarks( painter );
     }
 
+    if ( d->m_alternativeRoutesModel ) {
+        d->renderAlternativeRoutes( painter );
+    }
+
     if ( d->m_routingModel) {
         d->renderRoute( painter );
     }
 
-    if ( d->m_routeSkeleton) {
-        d->renderSkeleton( painter );
+    if ( d->m_routeRequest) {
+        d->renderRequest( painter );
     }
 
     painter->restore();
@@ -628,6 +734,14 @@ void RoutingLayer::synchronizeWith( QAbstractProxyModel *model, QItemSelectionMo
     d->m_proxyModel = model;
 }
 
+void RoutingLayer::synchronizeAlternativeRoutesWith( QComboBox *view )
+{
+    d->m_alternativeRoutesView = view;
+
+    connect( d->m_alternativeRoutesModel, SIGNAL( rowsInserted( QModelIndex, int, int) ),
+             this, SLOT( showAlternativeRoads() ) );
+}
+
 void RoutingLayer::setPointSelectionEnabled( bool enabled )
 {
     d->m_pointSelection = enabled;
@@ -645,21 +759,52 @@ void RoutingLayer::setRouteDirty( bool dirty )
     d->m_marbleWidget->repaint();
 }
 
-void RoutingLayer::setRouteSkeleton( RouteSkeleton *skeleton )
+void RoutingLayer::setRouteRequest( RouteRequest *request )
 {
-    d->m_routeSkeleton = skeleton;
+    d->m_routeRequest = request;
 }
 
 void RoutingLayer::removeViaPoint()
 {
     if ( d->m_activeMenuIndex >= 0 ) {
-        d->m_routeSkeleton->remove( d->m_activeMenuIndex );
+        d->m_routeRequest->remove( d->m_activeMenuIndex );
         d->m_activeMenuIndex = -1;
         setRouteDirty( true );
         emit routeDirty();
     }
 }
 
+void RoutingLayer::showAlternativeRoads()
+{
+    d->m_marbleWidget->repaint();
+}
+
+void RoutingLayer::exportRoute()
+{
+    QString fileName = QFileDialog::getSaveFileName( d->m_marbleWidget,
+                       tr( "Export Route" ), // krazy:exclude=qclasses
+                       QDir::homePath(),
+                       tr( "GPX and KML files (*.gpx *.kml)" ) );
+
+    if ( d->m_routingModel && !fileName.isEmpty() ) {
+        if ( fileName.endsWith( ".gpx", Qt::CaseInsensitive ) ) {
+            QFile gpx( fileName );
+            if ( gpx.open( QFile::WriteOnly) ) {
+                d->m_routingModel->exportGpx( &gpx );
+                gpx.close();
+            }
+        } else {
+            d->m_marbleWidget->model()->routingManager()->saveRoute( fileName );
+        }
+    }
+}
+
+void RoutingLayer::updateRouteState( RoutingManager::State state, RouteRequest *route )
+{
+    d->m_routeRequest = route;
+    setRouteDirty( state == RoutingManager::Downloading );
+}
+
 } // namespace Marble
 
 #include "RoutingLayer.moc"
