@@ -62,7 +62,7 @@ LanguagePropertiesPage::LanguagePropertiesPage(KEduVocDocument *doc, int identif
 
     // keyboard layout
     // try to talk to kxbk - get a list of keyboard layouts
-    QDBusInterface kxbk("org.kde.kxkb", "/kxkb", "org.kde.KXKB");
+    QDBusInterface kxbk("org.kde.keyboard", "/Layouts", "org.kde.KeyboardLayouts");
     QDBusReply<QStringList> reply = kxbk.call("getLayoutsList");
     if (reply.isValid()) {
         QStringList layouts = reply;
